##############################################################################
#
#   File:      al_command_line.py
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz & Olof Svensson
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/06/24
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005, AALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from al_string                  import ALString
from al_verbose                 import ALVerbose
from al_command                 import ALCommand
from al_dict                    import ALDict
from al_command_line_argument   import ALCommandLineArgument


class ALCommandLine( ALCommand ):
    """
    Base class for command line objects.
    """
    def __init__( self, _oalCommandLineArgument = None, _oalString = "objCommandLine" ):
        ALCommand.__init__( self, _oalString )
        self.m_oalDictCommand = ALDict()      
        if ( _oalCommandLineArgument is None ):
            self.m_oalCommandLineArgument = ALCommandLineArgument()
        else:
            self.m_oalCommandLineArgument = _oalCommandLineArgument.duplicate()
           
           
    def addCommand( self, _oalStringToken, _oalObjectMethod ):
        self.synchronizeOn()
        self.m_oalDictCommand.addObject( ALString( _oalStringToken ), _oalObjectMethod )
        self.synchronizeOff()
          
     
    def process( self ):
        self.synchronizeOn()
        if ( self.m_oalCommandLineArgument is not None ):
            for cmdkey in self.m_oalCommandLineArgument:
                if cmdkey in self.m_oalDictCommand:
                    oalObjectMethod = self.m_oalDictCommand[ cmdkey ]
                    oalObjectMethod( cmdkey )
        self.synchronizeOff()


    def existCommand( self, _oalString ):
        self.synchronizeOn()
        oalString = ALString( _oalString )
        bexist = self.m_oalCommandLineArgument.existArgument( oalString )
        self.synchronizeOff()
        return bexist
    
    
    def getArgument( self, _oalString ):
        self.synchronizeOn()
        oalString = ALString( _oalString )
        oalArgument = self.m_oalCommandLineArgument.getArgument( oalString )
        self.synchronizeOff()
        return oalArgument
    
    
    def addCommandArgument( self, _oalString ):
        self.synchronizeOn()
        oalString = ALString( _oalString )
        self.m_oalCommandLineArgument.append( oalString )
        self.synchronizeOff()



    def isEmpty( self ):
        self.synchronizeOn()
        bEmpty = False
        if len(self.m_oalCommandLineArgument)<=1:
            bEmpty = True
        self.synchronizeOff()
        return bEmpty
       

##############################################################################

