##############################################################################
#
#   File:      al_script_MD5Sum.py
#
#   Project:   Asynchronous Action Library Project
#              http://aalib.sourceforge.net/
#
#   Author(s): Romeu A. Pieritz & Olof Svensson
#   Contact  : aalib_info@yahoo.com
#
#   Date:      2005/06/24
#
#   License:
#   --------------------------------------------------------------------------
#   Asynchronous Action Library Project
#   Copyright (c) 2005, AALib developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:

#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of AALib's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from al_disk_explorer               import ALDiskExplorer
from al_string                      import ALString
from al_file                        import ALFile
from al_sys                         import ALSystem
from al_md5                         import ALMD5
from al_verbose                     import ALVerbose
from al_stream_file                 import ALStreamFile
from al_script                      import ALScript
from al_command_line_argument       import ALCommandLineArgument


class ALScriptMD5Sum( ALScript ):
   
                                                                           
    def evaluateMD5Sum( self, _oalStrFolder, _oalStrFileExtension ):
        bModified = False
        
        oalList =  ALDiskExplorer.getFolderList( _oalStrFolder )
        for ifile in oalList:
            if ALDiskExplorer.isFile( ifile ):
                if ALFile.isExtension( ifile, _oalStrFileExtension ):
                    oalMD5 = ALMD5()
                    oalStreamFile = ALStreamFile( ifile, "r" )
                    oalMD5.analyse( oalStreamFile.read() )
                    oalStreamFile.close()
                    if ( self.analyse( ifile, oalMD5.sum() ) ):
                        bModified = True
                        
        if (bModified):
            ALVerbose.screen( "Package::MODIFIED" )
        else:
            ALVerbose.screen( "Package::OK" )


    def analyse( self, _oalStrFile, _oalStrSum ):
        oalResSum = ALScriptMD5Sum.getResource( ALFile.getFileName( _oalStrFile ) )
        if (oalResSum == None):
            ALScriptMD5Sum.setResource( ALFile.getFileName( _oalStrFile ), _oalStrSum )
            ALVerbose.screen( "File::NEW      - " + _oalStrFile )
        else:
            oalStrSumOrig = ALString( oalResSum );
            if (oalStrSumOrig==_oalStrSum):
                ALVerbose.screen( "File::OK       - " + _oalStrFile )
            else:
                ALVerbose.screen( "File::MODIFIED - " + _oalStrFile )
                return True
        
        return False
    
    
    def reset( self, _oalObject = None ):
        ALScriptMD5Sum.resetResource()
    
  
    def process( self ):
        ALScriptMD5Sum.inputResource()
        self.evaluateMD5Sum( "../modules", ".py" )    
        #self.evaluateMD5Sum( "../../modules", ".py" ) # to test the compilated version

        
##############################################################################


if __name__ == '__main__':
     
    oalScript = ALScriptMD5Sum( "md5sum" )
    oalScript.addCommand( "--reset", oalScript.reset )
    oalScript.execute()


##############################################################################
