##############################################################################
#
#   File:      dr_.py
#
#   Project:   DataRanking Project
#
#   Author(s): Romeu A. Pieritz [pieritz@esrf.fr]
#              Olof Svensson [svensson@esrf.fr]
#
#   Date:      2005/06/24
#
#   License:
#   --------------------------------------------------------------------------
#   Data Ranking Project Project
#   Copyright (c) 2005, Data Ranking Project developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of DataRanking's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from dr_definition                  import *
from dr_kernel                      import DRVerbose
from dr_kernel                      import DRApplication
from dr_system                      import DRToken
from dr_system                      import DRString
from dr_system                      import DRList
from dr_system                      import DRXml
from dr_dna                         import DRDna


class DRDnaDataRanking( DRDna ):
    
    def __init__( self ):
        DRDna.__init__( self )
        self.m_rankValue        = 0
        self.m_rankEngine       = DRString( "RankEngine" )
        self.m_rankInformation  = DRString( DRApplication.getApplicationName() )
        
        
    def inputXmlDna( self, _odrXmlElement ):
        self.m_rankValue        = DRXml.getElementTokenValue( _odrXmlElement, DRToken( "rankValue" ) )
        self.m_rankEngine       = DRXml.getElementTokenValue( _odrXmlElement, DRToken( "rankEngine" ) )
        self.m_rankInformation  = DRXml.getElementTokenValue( _odrXmlElement, DRToken( "rankInformation" ) )
        self.m_rankValue        = self.m_rankValue.atof()
 

    def outputXmlDna( self, _odrXml, _odrXmlElement ):
        odrXmlElementObject = _odrXml.createElement( "dataRank" )
        _odrXmlElement.appendChild( odrXmlElementObject )
        
    
        odrXmlElement = _odrXml.createElement( DRToken( "rankValue" ))
        odrXmlElement.appendChild( _odrXml.createTextNode( DRString( self.m_rankValue ) ) )      
        odrXmlElementObject.appendChild( odrXmlElement )
        
     
        odrXmlElement = _odrXml.createElement( DRToken( "rankEngine" ))
        odrXmlElement.appendChild( _odrXml.createTextNode( DRString( self.m_rankEngine ) ) )      
        odrXmlElementObject.appendChild( odrXmlElement )
        
     
        odrXmlElement = _odrXml.createElement( DRToken( "rankInformation" ))
        odrXmlElement.appendChild( _odrXml.createTextNode( DRString( self.m_rankInformation ) ) )      
        odrXmlElementObject.appendChild( odrXmlElement )
        
      
##############################################################################
