##############################################################################
#
#   File:      dr_.py
#
#   Project:   DataRanking Project
#
#   Author(s): Romeu A. Pieritz [pieritz@esrf.fr]
#              Olof Svensson [svensson@esrf.fr]
#
#   Date:      2005/06/24
#
#   License:
#   --------------------------------------------------------------------------
#   Data Ranking Project Project
#   Copyright (c) 2005, Data Ranking Project developers (see above AUTHORS)
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
#   Neither the names of DataRanking's copyright owner nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##############################################################################


from dr_definition                  import *
from dr_kernel                      import DRVerbose
from dr_system                      import DRToken
from dr_system                      import DRString
from dr_system                      import DRList
from dr_system                      import DRXml
from dr_system                      import DRDict
from dr_dna                         import DRDna


class DRDnaScreeningStrategy( DRDna ):
    
    def __init__( self ):
        DRDna.__init__( self )     
        self.m_phiStart             = None
        self.m_phiEnd               = None
        self.m_rotation             = None
        self.m_exposureTime         = None
        self.m_resolution           = None
        self.m_program              = None
        self.m_ScreeningStrategyID  = None # int
        self.m_ScreeningOutputID    = None # int
                   
                   

    # Interface to setup values
    def setRotation( self, _fRotation ):
        self.m_rotation = (float)( _fRotation  )
    def getRotation( self ):
        return self.m_rotation


    def setExposureTime( self, _fExposureTime ):
        self.m_exposureTime = _fExposureTime
    def getExposureTime( self ):
        return self.m_exposureTime
        
        
    def setPhiInit( self, _fPhiInit ):
        self.m_phiStart = _fPhiInit 
    def getPhiInit( self ):
        return self.m_phiStart 
    
    
    def setPhiEnd( self, _fPhiEnd ):
        self.m_phiEnd = _fPhiEnd
    def getPhiEnd( self ):
        return self.m_phiEnd
        
        
        
#  <ScreeningStrategy>
#    <phiStart>29.000000</phiStart>
#    <rotation>1.000000</rotation>
#    <exposureTime>1.500000</exposureTime>
#    <resolution>1.180000</resolution>
#    <program>Mosflm</program>
#  </ScreeningStrategy>
  
  
  
    def inputXmlDna( self,  _odrXmlElement ): 
        for ikey in self.__dict__:
            odrValue = DRXml.getElementTokenValue( _odrXmlElement, DRXml.getAtributeToken( ikey ) )
            if (odrValue!=None):
                if ( ikey==DRToken( "m_program" ) ):
                    self.__dict__[ ikey ] = odrValue
                else:
                    self.__dict__[ ikey ] = odrValue.atof()
            

    def outputXmlDna( self, _odrXml, _odrXmlElement ):
        odrXmlElementObject = _odrXml.createElement( "ScreeningStrategy" )
        _odrXmlElement.appendChild( odrXmlElementObject )
        for ikey in self.__dict__:
            if ( ikey!=DRToken( "m_objectName" ) and ikey!=DRToken( "m_objectVersion" ) ):
                if ( ikey!=DRToken( "m_objectID" ) and ikey!=DRToken( "m_objectMagicNumber" ) ):
                    if ( self.__dict__[ ikey ]!= None ):         
                        odrXmlElement = _odrXml.createElement( DRXml.getAtributeToken( ikey ) )
                        odrXmlElement.appendChild( _odrXml.createTextNode( DRString( self.__dict__[ ikey ] ) ) )      
                        odrXmlElementObject.appendChild( odrXmlElement )
                
                
##############################################################################
